#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "readline.h"

typedef struct _Item {
    char *value;
    struct _Item *next;
} Item;

typedef struct {
    Item *first;
    Item *last;
} List;

void initList( List *l ) {
    l->first = NULL;
    l->last = NULL;
}

void pushFront( List *l, char *value ) {
    Item *newItem = (Item*) malloc( sizeof( Item ) );
    newItem->value = value;
    newItem->next = l->first;

    l->first = newItem;
    if ( !l->last )
        l->last = newItem;
}

void pushBack( List *l, char *value ) {
    Item *newItem = (Item*) malloc( sizeof( Item ) );
    newItem->value = value;
    newItem->next = NULL;
    
    if ( l->last )
        l->last->next = newItem;
    l->last = newItem;
    if ( !l->first )
        l->first = newItem;
}

void forEach( List *l, void(*callback)(char *) ) {
    Item *current = l->first;
    while ( current ) {
        callback( current->value );
        current = current->next;
    }
}

void printItem( char *value ) {
    printf( "[%s]\n", value );
}

void printList( List *l ) {
    forEach( l, printItem );
}
void freeChar( char *str ) {
    free( str );
}
void freeItemsInList( List *l ) {
    forEach( l, freeChar );
}

void clearList( List *l ) {
    Item *current = l->first;
    while ( current ) {
        Item *tmp = current;
        current = current->next;
        free( tmp );
    }
    l->first = NULL;
    l->last = NULL;
}

int main() {

    List l;
    initList( &l );

    puts( "write \"quit\" to end" );

    while ( 1 ) {

        printf( "> " );
        char *line = readLine();
        if ( strcmp( line, "quit" ) == 0 ) {
            free( line );
            break;
        }
        pushBack( &l, line );
    }

    printList( &l );
    freeItemsInList( &l );
    clearList( &l );
    return 0;
}
